package com.qmth.qrzk.dao.repositories;

import java.util.List;

import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;

import com.qmth.qrzk.batch.model.BatchGJCourse;

public interface BatchGJCourseRepo extends JpaRepository<BatchGJCourse, Long> {
	/**
	 * 通过批次id查找共卷课程
	 * @param batchId
	 * @return
	 */
	List<BatchGJCourse> findByBatchId(Integer batchId,Sort sort);
	
	/**
	 * 查找指定批次和课程代码的共卷信息
	 * @param batchId
	 * @param courseCode
	 * @return
	 */
	List<BatchGJCourse> findByBatchIdAndCourseCode(Integer batchId, String courseCode);
	
	/**
	 * 查找指定批次和被共卷课程代码的所有共卷信息
	 * @param batchId
	 * @param bCourseCode
	 * @return
	 */
	List<BatchGJCourse> findByBatchIdAndBCourseCode(Integer batchId, String bCourseCode);
	
	/**
	 * 删除批次下的所有共卷课程信息
	 * @param batchId
	 */
	@Modifying
	@Query("delete from BatchGJCourse where batchId = ?1")
	void deleteFromBatch(Integer batchId);
}
